; 
; Translucent dialog demo, with fade in, modeless dialog as main.
;
; Adapted from http://www.codeproject.com/dialog/smoothalpha.asp
;
; Requires Windows 2000 or later.
; 

; 
    include \masm32\include\masm32rt.inc
; 

    ; -------------------------------------------------------
    ; This extends the dialog macros to do modeless dialogs.
    ; A modeless dialog is needed here because modal dialogs
    ; are automatically made visible, and we need the dialog
    ; to start out invisible.
    ; -------------------------------------------------------

    CallModelessDialog MACRO Instance,Parent,DlgProc,lParamInit

      invoke CreateDialogIndirectParam,Instance,esi,Parent,
                                       ADDR DlgProc,lParamInit
      push eax                ;; preserve return value
      invoke GlobalFree,esi   ;; free memory
      pop eax                 ;; restore return value
      pop edi
      pop esi

    ENDM

; 

    DlgProc PROTO :DWORD,:DWORD,:DWORD,:DWORD

    .data?
      hDlg      dd ?
      hInstance dd ?
      msg       MSG <>
    .code

; 

start:

      mov hInstance, FUNC(GetModuleHandle,NULL)

      call main

      invoke ExitProcess,eax

; 

main proc

    ; ----------------------------------------------------
    ; For a modeless dialog, for the dialog to be visible
    ; the WS_VISIBLE style must be specified, but in this
    ; case we need the dialog to start out invisible
    ; ----------------------------------------------------

    Dialog "Translucent Dialog Demo", \     ; caption
           "MS Sans Serif",10, \            ; font,pointsize
            WS_OVERLAPPED or \              ; styles for
            WS_SYSMENU or DS_CENTER, \      ; dialog window
            1, \                            ; number of controls
            0,0,120,90, \                   ; x y co-ordinates
            1024                            ; memory buffer size

    DlgButton "Close",WS_TABSTOP,38,55,40,13,IDCANCEL

    CallModelessDialog hInstance,0,DlgProc,NULL
    mov hDlg, eax

    ; -----------------------------------------
    ; Unlike a modal dialog, a modeless dialog
    ; must have a message loop.
    ; -----------------------------------------

  msgLoop:

    invoke GetMessage,ADDR msg,0,0,0
    .IF (eax != 0)
      invoke IsDialogMessage,hDlg,ADDR msg
      jmp msgLoop
    .ENDIF

    ret

main endp

; 

DlgProc proc hWin:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD

    LOCAL bAlpha:DWORD

    Switch uMsg

      Case WM_INITDIALOG

        ; ---------------------
        ; Animation constants.
        ; ---------------------

        ANIMATION_MS      equ 250
        ALPHA             equ 200
        ALPHA_STEP        equ 10
        ANIMATION_TIMEOUT equ ALPHA_STEP * ANIMATION_MS / ALPHA

        ; ----------------------------------
        ; Make the dialog a layered window.
        ; ----------------------------------

        invoke GetWindowLong,hWin,GWL_EXSTYLE
        or eax, WS_EX_LAYERED
        invoke SetWindowLong,hWin,GWL_EXSTYLE,eax

        ; ----------------------------------------------------------
        ; Start at an alpha value of zero (completely transparent).
        ; ----------------------------------------------------------

        mov bAlpha, 0
        invoke SetLayeredWindowAttributes,hWin,0,bAlpha,LWA_ALPHA

        ; --------------------
        ; Display the dialog.
        ; --------------------

        invoke ShowWindow,hWin,SW_SHOW

        ; ------------------------------------------
        ; Force an update of the client area so the
        ; controls will be included in the fade in.
        ; ------------------------------------------

        invoke RedrawWindow,hWin,NULL,NULL,RDW_UPDATENOW

        ; ---------------------------------------
        ; Gradually fade the dialog in, stopping
        ; when the alpha value reaches ALPHA.
        ; ---------------------------------------

        .WHILE bAlpha < ALPHA
          invoke SetLayeredWindowAttributes,hWin,0,bAlpha,LWA_ALPHA
          invoke Sleep,ANIMATION_TIMEOUT
          add bAlpha,ALPHA_STEP
        .ENDW
        invoke SetLayeredWindowAttributes,hWin,0,ALPHA,LWA_ALPHA

        return 1

      Case WM_COMMAND

        Switch wParam

          Case IDCANCEL

            invoke DestroyWindow,hWin

        EndSw

      Case WM_DESTROY

        invoke PostQuitMessage,NULL

    EndSw

    return 0

DlgProc endp

; 

end start
